<?php

/**
*
* Trackback Config Class for MODx 0.9.5
*
*  Author  : soushi
*  Version : 1.01
*  Date    : 2007/9/7
*
*  License : GPL
*
*/

class TrackbackConfig{

  var $ConfigChunk = 'trackback';  // config file
  var $Config = array();           // config
  var $Allow  = array();           // Recv/Send/View Allow Status
  var $ErrMsg = '';                // Error Massage
  var $Table = 'site_trackback';   // Trackback table name
  var $LogLevel;                   // Log Level

  //Not use 'const' on php4....
  //Status
  var $ST_VIEW = 1;
  var $ST_NEW  = 2;

  //Log level
  var $L_NONE   = 0;
  var $L_ERROR  = 1; // Default
  var $L_NOTICE = 2;
  var $L_INFO   = 3;

  //Allow/Deny/Expire
  var $ckAllow  = '0';
  var $ckDeny   = '1';
  var $ckExpire = '2';


  //-----------------------------------------------------
  // Class Constructure
  // 
  // $fChunk    : string : Chunk name
  // $fLogLevel : int    : Log level
  // return     : none
  //-----------------------------------------------------
  function TrackbackConfig($fChunk='',$fLogLevel=''){
    global $modx;

    $this->LogLevel = $fLogLevel == ''?$this->L_ERROR:$fLogLevel;
    $this->ConfigChunk = $fChunk != '' ? $fChunk :$this->ConfigChunk;
    //add prefix
    $this->Table = $modx->db->config['table_prefix'] . $this->Table;

    //Config Setting
    $this->SetDefaultConfig();
    $this->SetConfigFromChunk($this->ConfigChunk);

    //if(extension_loaded("mbstring")){
    //  $this->mbEnable = true;
    //}

  } 

  //-----------------------------------------------------
  //  Setting Default Config
  //
  //  return  : none
  //-----------------------------------------------------
  function SetDefaultConfig(){
    $this->Config = array();

    $this->Config['EnableSendAlert']          = true;
    $this->Config['SendAlertUser']            = 1;
    $this->Config['ReceiveDocID']             = 1;
    $this->Config['DocAuthor']                = 'myself';
    $this->Config['OverwriteDocAuthor']       = true;
    $this->Config['AllowDocID_Recv']       = '';
    $this->Config['AllowDocID_Send']       = '';
    $this->Config['AllowDocID_View']       = '';
    $this->Config['EnableNowOpen']            = false;
    $this->Config['NowOpenExceptOpenDomain']  = true;
    $this->Config['NowOpenExceptDocID']       = '';
    $this->Config['DenyHideTb']               = false;
    $this->Config['DenyDoubleTb']             = false;
    $this->Config['DenyThisTb']               = '';
    $this->Config['CheckOnSendPingFromTV']    = false;
    $this->Config['CheckOnSendPingAutomatic'] = false;
    $this->Config['ViewTpl']                  = '';
    $this->Config['ViewTplTbList']            = '';
    $this->Config['EmptyText']                = 'No Trackback';

    //return true;
  }

  //-----------------------------------------------------
  //  Setting Config from Chunk
  //
  //  $fChunk  : string : chunk name
  //  return   : bool : true/false
  //-----------------------------------------------------
  function SetConfigFromChunk($fChunk=''){
    global $modx;

    $fChunk = $fChunk != '' ? $fChunk : $this->ConfigChunk;

    //$cnks = $modx->getChunk($fChunk); // no use,Because cash is poor. 
    $tbl = $modx->db->config['table_prefix'] . 'site_htmlsnippets';
    $whr = "name = '$fChunk'";

    if(! $rs = $modx->db->select("snippet",$tbl,$whr)){
      $this->Pnt('Config:SetConfigFromChunk','Chunk load failed:'.$fChunk,$this->L_ERROR);
      return false;
    }
    $resource= $modx->fetchRow($rs);
    $cnks= $resource['snippet'];

    $this->Pnt('Config:SetConfigFromChunk-Chunk',
	       'cnkname='.$fChunk.'<br />cnks='.$cnks,
	       $this->L_INFO);
    $cnks = explode("\n",$cnks);
    $this->Pnt('Config:SetConfigFromChunk-ExplodeChunk',
	       'cnkname='.$fChunk.'<br />cnks='.print_r($cnks,true),
	       $this->L_INFO);

    foreach ($cnks as $cnk){
      if( preg_match('/^[ \t]*#/',$cnk) == 1 ){continue;} //comment

      $tmp = preg_split("/[ \t]*=[ \t]*/",$cnk,2);
      $cnfs["$tmp[0]"] = $tmp[1];
    }
    $this->Pnt('Config:SetConfigFromChunk-cnfs',
	       'cnfs='.print_r($cnfs,true),
	       $this->L_INFO);

    $eflg = false;
    $msg = '';
    foreach ($cnfs as $key =>$val){
      if( $this->SetValue($key,$val,false) == false){
	$msg .= $this->ErrMsg;
	$eflg = true;
      }
    }
    if( $eflg ){ $this->ErrMsg = $msg;}    
    return !$eflg;
  }

  //-----------------------------------------------------
  //  Setting Config from POST method
  //
  //  return   : bool : true/false
  //-----------------------------------------------------
  function SetConfigFromPost(){
    global $modx;

    $eflg=false;
    $msg='';
    foreach ($_POST as $key =>$val){
      if( $this->SetValue($key,$val,false) == false){
	$msg .= $this->ErrMsg;
	$eflg = true;
      }
    }
    if( $eflg ){ $this->ErrMsg = $msg;}

    return !$eflg;
  }

  //-----------------------------------------------------
  //  Set config(one param)
  //
  //  $fParam      : string : config param
  //  $fData       : string : config data
  //  $fNoMatchErr : bool   : check no match $fParam 
  //  return  : bool : true/false
  //-----------------------------------------------------
  function SetValue($fParam,$fData,$fNoMatchErr=true){
    global $_lang;

    switch ($fParam){

    case 'EnableSendAlert':
    case 'OverwriteDocAuthor':
    case 'EnableNowOpen':
    case 'NowOpenExceptOpenDomain':
    case 'DenyHideTb':
    case 'DenyDoubleTb':
    case 'CheckOnSendPingFromTV':
    case 'CheckOnSendPingAutomatic':
      if( $fData == true || $fData == 1 ){
	$this->Config[$fParam] = true;
      }else{
	$this->Config[$fParam] = false;
      } 

      break;

    case 'ReceiveDocID':
      $this->Config[$fParam] = $fData;
      if( !(is_numeric($fData) &&
	    $fData > 0)){
	$this->ErrMsg = $_lang['tbc_error_recid']."\n";
	return false;
      }
      break;

    case 'AllowDocID_Recv':
    case 'AllowDocID_Send':
    case 'AllowDocID_View':
    case 'NowOpenExceptDocID':

      $this->Config[$fParam] = $fData;
      if( $fData <> '' && 
	  preg_match('/^([0-9]+[Rrn]*(c[0-9]+)*[Rrn]*,)*[0-9]+[Rrn]*(c[0-9]+)*[Rrn]*$/',$fData) == 0 ){
	$this->ErrMsg = $_lang[$fParam] .' '. $_lang['tbc_error_allowdocid']."\n";
	return false;
      }
      break;

    case 'SendAlertUser':
    case 'DocAuthor':
    case 'DenyThisTb':
    case 'ViewTpl':
    case 'ViewTplTbList':
    case 'EmptyText':
      $this->Config[$fParam] = $fData;

      break;

    default:
      //no match
      if( $fNoMatchErr == true ){
	$this->ErrMsg = 'No match Param:'.$fParam."\n";
	return false;
      }else{
	return true;
      }
    }
    
    return true;

  }

  //-----------------------------------------------------
  //  Get config
  //
  //  $fParam : string : config param
  //  return  : any    : config data/false
  //-----------------------------------------------------
  function GetValue($fParam){

    if( isset($this->Config[$fParam])){
      return $output = $this->Config[$fParam];
    }
    return false;
  }

  //-----------------------------------------------------
  //  Get Template
  //
  //  $fType : string : template type
  //  return : string : template
  //-----------------------------------------------------
  function GetTpl($fType){
    global $modx;
    
    $output = $modx->getChunk($this->Config[$fType]);
    if( $output == "" ){
      switch ($fType){
      case 'ViewTpl':
	$output =<<<_
<div class="tb">
  TrackbackURL:[+tburl+]<br />
  <br />
  [+tbrdf+]
  [+tblist+]
</div>
_;

	break;
      case 'ViewTplTbList':
	$output =<<<_
<div class="tb_row">
  <div class="tb_num">[+tbnumber+]</div>
  <div class="tb_title">[+tbtitle+]</div>
  <div class="tb_excerpt">[+tbexcerpt+]</div>
  <span class="tb_author"><a href="[+tburl+]">[+tbblogname+]</a></span>
  <span class="tb_date">on [+tbdate+]</span>
</div>
_;
	break;

      }
    }
    return $output;
  }

  //-----------------------------------------------------
  //  Get Author
  //
  //  $userid : integer  : user id
  //  return  : string   : author name
  //-----------------------------------------------------
  function GetAuthor($userid=0){
    global $modx;

    if( $this->Config['OverwriteDocAuthor'] == ture ){
      $output = $this->Config['DocAuthor'];
    }else{
      $rs = $modx->db->select("fullname",
			      $modx->getFullTableName("user_attributes"),
			      "id = " .$userid,
			      "id DESC",
			      "1");
      $resource = $modx->fetchRow($rs); 
      if( $resource['fullname'] == '' ){
	$output = $this->Config['DocAuthor'];
      }else{
	$output = $resource['fullname'];
      }
    }
    return $output;
  }

  //-----------------------------------------------------
  //  Save Config To Chunk
  //
  //  return  : bool   : true/false
  //-----------------------------------------------------
  function SaveConfigToChunk(){
    global $modx;

    $configdata='# trackback tool config' . "\n";

    foreach($this->Config as $key => $val) {
      if( $val === true ){
	$val = 1;
      }elseif( $val === false ){
	$val = 0;
      }else{
	$val = $modx->db->escape($val);
      }
      $configdata .= $key . '=' . $val . "\n";
    }

    //save to chunk...
    $fld = "snippet = '$configdata'";
    $tbl = $modx->db->config['table_prefix'] . 'site_htmlsnippets';
    $whr = "name = '$this->ConfigChunk'";
    if(! $modx->db->update( $fld , $tbl,$whr) ){
      Pnt('Config:SaveConfigToChunk','DB update failed.',$this->L_ERROR);
      return false;
    }

    return true;
  }

  //-----------------------------------------------------
  //  Check Allow View
  //
  //  $fDocID : int : Document ID
  //  return  : int    : 0..ok,1..ng,2..expire
  //-----------------------------------------------------
  function CheckAllowView($fDocID){
    if(! isset($this->Allow['View'][$fDocID])){
      $this->Allow['View'][$fDocID] = $this->CheckAllow($fDocID,$this->Config['AllowDocID_View']);
    }
    return $this->Allow['View'][$fDocID];

  }

  //-----------------------------------------------------
  //  Check Allow Send
  //
  //  $fDocID : int : Document ID
  //  return  : int    : 0..ok,1..ng,2..expire
  //-----------------------------------------------------
  function CheckAllowSend($fDocID){
    if(! isset($this->Allow['Send'][$fDocID])){
      $this->Allow['Send'][$fDocID] = $this->CheckAllow($fDocID,$this->Config['AllowDocID_Send']);
    }
    return $this->Allow['Send'][$fDocID];
  }

  //-----------------------------------------------------
  //  Check Allow Recv
  //
  //  $fDocID : int : Document ID
  //  return  : int    : 0..ok,1..ng,2..expire
  //-----------------------------------------------------
  function CheckAllowRecv($fDocID){
    if(! isset($this->Allow['Recv'][$fDocID])){
      $this->Allow['Recv'][$fDocID] = $this->CheckAllow($fDocID,$this->Config['AllowDocID_Recv']);
    }
    return $this->Allow['Recv'][$fDocID];
  }

  //-----------------------------------------------------
  //  Hide Except
  //
  //  $fDocID : int : Document ID
  //  return  : int    : 0..ok,1..ng,2..expire
  //-----------------------------------------------------
  function HideExcept($fDocID){
    if(! isset($this->Allow['Hide'][$fDocID]) &&
       $this->Config['NowOpenExceptDocID'] != ''){
      $this->Allow['Hide'][$fDocID] = $this->CheckAllow($fDocID,$this->Config['NowOpenExceptDocID']);
    }
    return $this->Allow['Hide'][$fDocID];
  }

  //-----------------------------------------------------
  //  Check Allow Recv/Send/View/ExceptShow
  //
  //  $fDocID : int    : Document ID
  //  $fList  : string : Access Lists
  //  return  : int    : 0..ok,1..ng,2..expire
  //-----------------------------------------------------
  function CheckAllow($fDocID,$fList){
    global $modx;
    global $lang;

    if( $fList == '' ){ return $this->ckAllow;}

    $docObj = $modx->getDocumentObject('id',$fDocID);
    $docIDs = Array();
    $docIDs['pid'][$fDocID] = $docObj['parent'];
    $docIDs['ctime'][$fDocID] = $docObj['createdon'];
    $rslt=$this->ckDeny;

    $lists = explode(',',$fList);
    foreach($lists as $key => $val){
      if(! preg_match('/^[0-9]+/',$val,$pid)){
        $this->Pnt('Config:CheckAllow','Allow ParentID not found:'.$val,$this->L_ERROR);
      }else{
	$pid = $pid[0];
	$rsvL = strpos($val,'R') === false ? false : true;
	$rsvS = strpos($val,'r') === false ? false : true;
	$ngtv = strpos($val,'n') === false ? false : true;

	if( preg_match('/c([0-9]+)/',$val,$expire)){
	  $expire = $expire[1] * 60 * 60 * 24;
	}else{
	  $expire = 0;
	}
      }

      if( $expire <> 0 && 
	  (time() - $docIDs['ctime'][$fDocID]) > $expire){
	$rslt=$this->ckExpire;
      }else{
	$i=$fDocID;
	while( $i != 0){
	  if( ($rsvL == true && $docIDs['pid'][$i] == $pid) ||
	      ($rsvS == true && $i == $pid) ||
	      (($rsvL != true && $rsvS != true) && $fDocID == $pid) ||
	      (($rsvL == true || $rsvS == true) && $pid == 0)){
	    
	    $rslt= $ngtv == true ? $this->ckDeny : $this->ckAllow;
	    
	    break;
	  }	  
	  // next document
	  $i = $docIDs['pid'][$i];
	  if( $i != 0 && ! isset($docIDs['pid'][$i])){
	    $docObj = $modx->getDocumentObject('id',$i);
	    $docIDs['pid'][$i] = $docObj['parent'];
	    $docIDs['ctime'][$i] = $docObj['createdon'];
	  }
	}
      }

    }

    $this->Pnt('Config:CheckAllow','DocID='.$fDocID.':Result='.$rslt,$this->L_INFO);
    return $rslt;

  }

  //-----------------------------------------------------
  //  Database initialisation
  //
  //  return  : bool   : true/false
  //-----------------------------------------------------
  function DBInit(){
    global $modx;
    global $lang;

    $q = 'DROP TABLE IF EXISTS ' . $this->Table;
    if(! $modx->db->query($q)){
      $this->ErrMsg = $_lang['tbc_tb_drop_ng'];
      return false;
    }

    $q = 
      'CREATE TABLE `'.$this->Table.'` ('.
      '  `id`        int(11)      NOT NULL auto_increment,'.
      '  `parent_id` int(11)      NOT NULL default \'0\' ,'.
      '  `domain`    varchar(255) NOT NULL               ,'.
      '  `remoteip`  varchar(15)  NOT NULL               ,'.
      '  `url`       varchar(255) NOT NULL               ,'.
      '  `title`     varchar(255) NOT NULL default \'\'  ,'.
      '  `excerpt`   text         NOT NULL               ,'.
      '  `blog_name` varchar(255) NOT NULL default \'\'  ,'.
      '  `date`      int(14)      NOT NULL default \'0\' ,'.
      '  `status`    tinyint(1)   NOT NULL default \'0\' ,'.
      'PRIMARY KEY  (`id`)'.
      ') ENGINE=MyISAM;';

    if(! $modx->db->query($q)){
      $this->ErrMsg = $_lang['tbc_tb_create_ng'];
      return false;
    }
    return true;

  }

  //-----------------------------------------------------
  //  Output to MODx log
  //
  //  $fTitle    : string  : message title
  //  $fMsg      : string  : message
  //  $fLogLevel : integer : Log level
  //  return : none   :
  //-----------------------------------------------------
  function Pnt($fTitle='-',$fMsg,$fLogLevel=''){
    global $modx;

    $fLogLevel == '' && $fLogLevel = $this->L_ERROR;
    if( $fLogLevel <= $this->LogLevel ){
      $modx->logEvent(1, 1, $fMsg, 'Trackback:'.$fTitle);
    }
  } 

}
?>
