<?php

/**
*
* Trackback Core Class for MODx 0.9.5
*
*  Original author : Ran Aroussi <ran@blogish.org> 
*  2nd      author : Florian Leifeld (info@florian-leifeld.de)
*
*  Author  : soushi
*  Version : 1.08
*  Date    : 2007/9/10
*
*  License : GPL
*
*/

require_once('tbconfig.php');

class Trackback{

  var $Cnf;                      // Config data
  var $mbEnable = false;          // multibyte string function support
  var $DocID = '';                // Target Document ID
  var $Charset = ''; // Default encoding used throughout the class (ie. UTF-8)
  var $ErrMsg = '';

  var $excerptContentSize = 255; // excerpt string length(except introtext)
  var $StringSize = 255;         // URL,blog_name,title string length

  var $DEBUG;             // DEBUG MODE
  var $VERSION = '2.00';          // This resource version

  //Not use 'const' on php4....
  //Allow/Deny/Expire
  var $ckAllow  = '0';
  var $ckDeny   = '1';
  var $ckExpire = '2';

  //-----------------------------------------------------
  // Class Constructure
  // 
  // $fCharset  : string : text encode
  // $fLogLevel : int    : Log Level
  // return : none
  //-----------------------------------------------------
  function Trackback($fChunk='trackback',$fCharset = "UTF-8",$fLogLevel=''){
    //global $modx;

    $this->Charset = $fCharset;
    $this->Cnf     = new TrackbackConfig($fChunk,$fLogLevel);
    $this->LogLevel = $fLogLevel == '' ? $this->Cnf->L_ERROR : $fLogLevel;

    if(extension_loaded("mbstring")){
      $this->mbEnable = true;
    }

  } 

  //-----------------------------------------------------
  //  Get Value
  //  $fKey  : string : key
  //  return : mix    : value
  //-----------------------------------------------------
  function GetValue($fKey){

    return $this->Cnf->GetValue($fKey);

  } 

  //-----------------------------------------------------
  //  Set Value
  //  $fKey  : string : key
  //  $fVal  : string : value
  //  return : bool 
  //-----------------------------------------------------
  function SetValue($fKey,$fVal){
    global $_lang;

    if( $fKey == 'DocID' ){
      if( !is_numeric($fVal) || $fVal < 1 ){
	$this->ErrMsg .= $_lang['tb_error_docid'];
	return false;
      }
      $this->DocID = $fVal;
    }else{
      if(! $this->Cnf->SetValue($fKey,$fVal)){
	$this->ErrMsg .= $this->Cnf->ErrMsg;
	return false;
      }
    }
    return true;

  } 

  //-----------------------------------------------------
  //  Action
  //  $fMode : string : action mode
  //  return : string : HTML
  //-----------------------------------------------------
  function Action($fMode){
    $output = '';

    switch ($fMode) {
      //Count trackbacks
    case 'count':
      $output = $this->TrackbackCount($this->DocID);
      break;
    
      //Receive trackback from other blog
    case 'receive':
      $output = $this->ReceiveTrackback();
      break;
    
      //Show RDF and trackbacks 
    default:
      $output= $this->MakeTrackback($this->DocID);
      
    }

    return $output;

  } 



  //-----------------------------------------------------
  //  Count Trackbacks
  //
  //  $fDocID : integer : Document ID
  //  return  : integer : trackback count(blank is error)
  //-----------------------------------------------------
  function TrackbackCount($fDocID = 0){
    global $modx;

    if( $this->Cnf->CheckAllowView($fDocID) != $this->ckAllow){
      return '';
    }

    if(! $rs = $modx->db->select("id",
				 $this->Cnf->Table,
				 "     parent_id = ".$fDocID.
				 " and status & " . $this->Cnf->ST_VIEW)){
      $this->Cnf->Pnt('TrackbackCount','DB Query failed.',$this->Cnf->L_ERROR);
      return '';
    }

    $cnt=0;
    while ($row= $modx->db->getRow($rs)) {
      $cnt++;
    }

    return $cnt;

  } 

  //-----------------------------------------------------
  //  Make RDF(auto-discovery)
  //
  //  $fDocID : integer : document id
  //  return  : strings :
  //-----------------------------------------------------
  function MakeRDF($fDocID = 0){
    global $modx,$_lang;

    $recID = $this->Cnf->GetValue('ReceiveDocID');

    $rslt =  $this->Cnf->CheckAllowRecv($fDocID);
    switch ($rslt) {
    case $this->ckDeny:
      return '<!-- '.$_lang['tb_recv_deny'].'-->';
      break;
    case $this->ckExpire:
      return '<!-- '.$_lang['tb_recv_expire'].'-->';
      break;
    }

    $docObj = $modx->getDocumentObject('id',$fDocID);

    $rdf_title   = $docObj['pagetitle'];
    if( $docObj['introtext'] != "" ){
      $rdf_excerpt = $modx->stripTags($docObj['introtext']);
    }else{
      $rdf_excerpt = $this->CutText($modx->stripTags($docObj['content']),$this->excerptContentSize,true);
    }

    $rdf_date = $this->TimeToW3CDTF($docObj['createdon']);
    $rdf_link = $modx->makeUrl($fDocID,'','','full');
    $rdf_ping = $modx->makeUrl($recID,'','tb_id=' . $fDocID,'full');

    $rdf_author = $this->Cnf->GetAuthor($docObj['createdby']);

    $output = "<!-- \n"
      . "<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" \n"
      . "	xmlns:dc=\"http://purl.org/dc/elements/1.1/\" \n"
      . "	xmlns:trackback=\"http://madskills.com/public/xml/rss/module/trackback/\"> \n"
      . "<rdf:Description \n"
      . "	rdf:about=\"" . $this->xml_safe($rdf_link) . "\" \n"
      . "	dc:identifier=\"" . $this->xml_safe($rdf_link) . "\" \n"
      . "	trackback:ping=\"" . $this->xml_safe($rdf_ping) . "\" \n"
      . "	dc:title=\"" . $this->xml_safe($rdf_title) . "\" \n"
      . "	dc:subject=\"TrackBack\" \n"
      . "	dc:description=\"" . $this->xml_safe($rdf_excerpt) . "\" \n"
      . "	dc:creator=\"" . $this->xml_safe($rdf_author) . "\" \n"
      . "	dc:date=\"" . $rdf_date . "\" /> \n"
      . "</rdf:RDF> \n"
      . "-->  \n";

    return $output;

  } 

  //-----------------------------------------------------
  //  Make trackback URL
  //
  //  $fDocID     : integer : document id
  //  return     : strings : 
  //-----------------------------------------------------
  function MakeURL($fDocID = 0){
    global $modx,$_lang;

    $rslt =  $this->Cnf->CheckAllowRecv($fDocID);
    switch ($rslt) {
    case $this->ckDeny:
      $output = $_lang['tb_recv_deny'];
      break;
    case $this->ckExpire:
      $output = $_lang['tb_recv_expire'];
      break;
    default:
      $output = $modx->makeUrl($this->Cnf->GetValue('ReceiveDocID'),'','tb_id=' . $fDocID,'full');
    }

    return $output;

  }

  //-----------------------------------------------------
  //  Make tackback HTML
  //
  //  $fDocid    : integer : document id
  //  return     : strings : HTML code
  //-----------------------------------------------------
  function MakeTrackback($fDocID){
    global $modx,$_lang;

    if( $this->Cnf->CheckAllowView($fDocID) != $this->ckAllow){
      return $_lang['tb_view_deny'];
    }

    $tpl    = $this->Cnf->GetTpl('ViewTpl');
    $output = $tpl;
    if(! (strpos($tpl,'[+tburl+]') === false) ){
      $output = str_replace('[+tburl+]',$this->MakeURL($fDocID),$output);
    }

    if(! (strpos($tpl,'[+tbrdf+]') === false) ){
      $output = str_replace('[+tbrdf+]',$this->MakeRDF($fDocID),$output);
    }

    if(! (strpos($tpl,'[+tblist+]') === false) ){
      $output = str_replace('[+tblist+]',$this->MakeTrackbackList($fDocID),$output);
    }

    return $output;
  }

  //-----------------------------------------------------
  //  Make received trackback list
  //
  //  $fDocID    : integer : document id
  //  return     : strings : 
  //-----------------------------------------------------
  function MakeTrackbackList($fDocID = 0){
    global $modx,$_lang;

    if(! $rs = $modx->db->select("url, title, blog_name, excerpt, date",
				 $this->Cnf->Table,
				 "     parent_id = ".$fDocID.
				 " and status & " . $this->Cnf->ST_VIEW,
				 "date DESC")){
      return $_lang['tb_fail_query'];
    }

    $i=0;
    $output = '';
    $tpl = $this->Cnf->GetTpl('ViewTplTbList');
    while ($row= $modx->db->getRow($rs)) {

      $i++;
      $tmptpl = $tpl;
      $tmptpl = str_replace('[+tbnumber+]',$i,$tmptpl);
      $tmptpl = str_replace('[+tbtitle+]',$row['title'],$tmptpl);
      //$shortExcerpt = iconv("UTF-8","ISO-8859-1",$resource[$i]['excerpt']);
      //$tmptpl = str_replace('[+tbexcerpt+]',$shortExcerpt,$tmptpl);
      //$tmptpl = str_replace('[+tbexcerpt+]',$row['excerpt'],$tmptpl);
      $tmptpl = str_replace('[+tbexcerpt+]',str_replace("\n","<br />\n",$row['excerpt']),$tmptpl);
      $tmptpl = str_replace('[+tburl+]',$row['url'],$tmptpl);
      $tmptpl = str_replace('[+tbblogname+]',$row['blog_name'],$tmptpl);
      $tmptpl = str_replace('[+tbdate+]',strftime("%Y/%m/%d %T",$row['date']),$tmptpl);

      $output .= $tmptpl;
    }
    if( $output == '' ){
      $output = $this->Cnf->GetValue('EmptyText');
    }
    return $output;

  }

  //-----------------------------------------------------
  //  Recive Trackback from other blog.
  //
  //  return : string : result
  //-----------------------------------------------------
  function ReceiveTrackback(){
    global $modx,$_lang;

    $errmsg = '';
    $data = array();
    $match = array();
    
    // get trackback target
    if( ! isset($_GET['tb_id']) || !is_numeric($_GET['tb_id']) ) {
      $errmsg = "Unknown Target Document ID\n";
      $this->Cnf->Pnt('ReceiveTrackback',
		      "Failed Trackback Receive.\n\n".
		      "Error:".$errmsg,
		      $this->Cnf->L_NOTICE);
      return $this->MakeResponseXML(false,$errmsg);
    }
    $data['docid'] = $_GET['tb_id'];

    //First access Check
    $access=true;
    if( $this->Cnf->CheckAllowRecv($data['docid']) != $this->ckAllow){
      $errmsg = 'This document not allow trackback. ID:'.$data['docid']."\n";
      $access=false;
    }else{
      // Get URL
      if( ! isset($_POST['url'])) {
	$errmsg .= 'No Input URL.'."\n";
      }else{
	$data['url'] = $this->CutText($_POST['url'],$this->StringSize);
	$data['domain'] = parse_url($data['url'],PHP_URL_HOST);
      }

      // Get Title
      $data['title'] = isset($_POST['title']) ? $this->CutText($_POST['title'],$this->StringSize) : $url;
      $data['excerpt'] = isset($_POST['excerpt']) ? $this->CutText($_POST['excerpt'],$this->excerptContentSize,true) : '';
      $data['blog_name'] = isset($_POST['blog_name']) ? $this->CutText($_POST['blog_name'],$this->StringSize) : 'no name';
      
      if( isset($_SERVER['HTTP_X_FORWARDED_FOR']) && 
	  preg_match('/^(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/',$_SERVER['HTTP_X_FORWARDED_FOR'],$match) != 0 ){
	$data['remoteip'] = $match[0];
      }else{
	$data['remoteip'] = $_SERVER['REMOTE_ADDR'];
      }

      //status(view and new)
      $data['status'] = $this->Cnf->ST_NEW;
      if( $this->Cnf->GetValue('EnableNowOpen') == false){
	$data['status'] = $data['status'] | $this->Cnf->ST_VIEW;
      }else{
	if( $this->Cnf->GetValue('NowOpenExceptOpenDomain') == true ){
	  if(! $rs = $modx->db->select("id",
				       $this->Cnf->Table,
				       "     domain = '".$data['domain']."'".
				       " and status & " . $this->Cnf->ST_VIEW)){
	    $errmsg .= 'DB Query failed.'."\n";
	  }elseif($modx->db->getRow($rs)){
	    $data['status'] = $data['status'] | $this->Cnf->ST_VIEW;
	  }
	}
	if( $this->Cnf->HideExcept($data['docid']) == $this->ckAllow){
	  $data['status'] = $data['status'] | $this->Cnf->ST_VIEW;
	}
      }
       
      //Second access Check
      if( $this->Cnf->GetValue('DenyHideTb') == true ){
	if(! $rs = $modx->db->select("id",
				     $this->Cnf->Table,
				     "     domain = '".$data['domain']."'".
				     " and not status & " . $this->Cnf->ST_VIEW)){
	  $errmsg .= 'DB Query failed.'."\n";
	}elseif($modx->db->getRow($rs)){
	  $errmsg = 'Not allow Trackback from your domain. ID:'.$data['docid']."\n";
	  $access=false;
	}
      }
      if( $this->Cnf->GetValue('DenyDoubleTb') == true ){
	if(! $rs = $modx->db->select("id",
				     $this->Cnf->Table,
				     "     domain = '".$data['domain']."'".
				     " and parent_id = " .$data['docid'])){
	  $errmsg .= 'DB Query failed.'."\n";
	}elseif($modx->db->getRow($rs)){
	  $errmsg = 'Not allow Trackback.Double message. ID :'.$data['docid']."\n";
	  $access=false;
	}
      }
      $domains = explode(',',$this->Cnf->GetValue('DenyThisTb'));
      foreach($domains as $key => $val){
	if( $val != "" && $val == $data['domain'] ){
	  $errmsg = 'Not allow trackback by Access List. ID:'.$data['docid']."\n";
	  $access=false; 
	}      
      }

    }

    // Access deny
    if( $access == false ){
      $this->Cnf->Pnt('ReceiveTrackback-Deny',
		      $errmsg,
		      $this->Cnf->L_ERROR);
      return $this->MakeResponseXML(false,$errmsg);
    }
    
    
    foreach($data as $key => $val) {
      $data[$key] = $modx->stripTags($modx->db->escape($data[$key]));
      //$data[$key] = $modx->db->escape($data[$key]);
    }
    
    if( $errmsg == '' ){
      $fields = array('id'        => '',
		      'parent_id' => $data['docid'],
		      'domain'    => $data['domain'],
		      'remoteip'  => $data['remoteip'],
		      'url'       => $data['url'],
		      'title'     => $data['title'],
		      'excerpt'   => $data['excerpt'],
		      'blog_name' => $data['blog_name'],
		      'status'    => $data['status'],
		      'date'      => time()
		      );
      
      if(! $modx->db->insert( $fields , $this->Cnf->Table ) ){
        $errmsg = 'Sorry,Trackback failed(system error).'."\n";      
	$this->Cnf->Pnt('ReceiveTrackback-failed',
			$errmsg.' ID:'.$data['docid'],
			$this->Cnf->L_ERROR);
	return $this->MakeResponseXML(false,$errmsg);
      }
    }

    $this->Cnf->Pnt('ReceiveTrackback-Success',
		    'Successed Trackback Receive:'.$data['url'],
		    $this->Cnf->L_NOTICE);

    if( $this->Cnf->GetValue('EnableSendAlert') == true){
      //$docObj = $modx->getDocumentObject('id',$data['docid']);
      //$user   = $docObj['createdby'] > 0 ? $docObj['createdby'] : 1;

      $modx->sendAlert('alert',$this->Cnf->GetValue('SendAlertUser'),1,
		       $_lang['tb_alert_title'],
		       $_lang['tb_alert_body1'].
		       $_lang['tb_alert_body2'].$data['url']."\n".
		       $_lang['tb_alert_body3'].$modx->makeUrl($data['docid'],'','','full')."\n",1);
    }

    return $this->MakeResponseXML(true);

  } 

  //-----------------------------------------------------
  //  Output from data in OnDocFormRender event
  //
  //  $fDocID   : integer : Document ID
  //  return    : string  : formdata
  //-----------------------------------------------------
  function GetForm($fDocID){
    global $_lang;

    $output = '';
    if( empty($fDocID) || $this->Cnf->CheckAllowSend($fDocID) == $this->ckAllow){
      $tvPing='';
      $autoPing='';
      if( $this->Cnf->GetValue('CheckOnSendPingFromTV') == true ){
	$tvPing = 'checked';
      }
      if( $this->Cnf->GetValue('CheckOnSendPingAutomatic') == true ){
	$autoPing='checked';
      }

      $output = 
	'<div class="sectionHeader">&nbsp;'.$_lang['tb_form_title']. '</div>'."\n".
	'<div class="sectionBody">'."\n".
	'  <input type="checkbox" name="tb_enableTVPing" '.$tvPing.' />&nbsp;'.$_lang['tb_form_tvping'].'<br />'."\n".
	'  <input type="checkbox" name="tb_enableAutoPing" '.$autoPing.' />&nbsp;'.$_lang['tb_form_autoping'].'<br />'."\n".
	'</div>'."\n";
    }
    return $output;

  }

  //-----------------------------------------------------
  //  Unix timestamp to WECDTF format
  //
  //  $time  : integer  : Unix timestamp
  //  return : string   : WECDTF or Error message
  //-----------------------------------------------------
  function TimeToW3CDTF($time){
    global $_lang;
    if( is_numeric($time)){
      return strftime("%Y-%m-%dT%T",$time);
    }
    return $_lang['invalid_timestamp'];
  }

  //-----------------------------------------------------
  //  Sending trackback ping
  //
  //  $fDocID : integer : document id
  //  $fTbURL  : string  : trackback URL
  //  return  : bool    : true/false
  //-----------------------------------------------------
  function ping($fDocID,$fTbURL){
    global $modx;
    global $_lang;

    $data  = '';
    $target= '';
    $response = '';

    $this->Cnf->Pnt('Ping - Start','Ping Process Start.',$this->Cnf->L_INFO);

    if( $fTbURL == '' || 
	! is_numeric($fDocID)){
      $this->Cnf->Pnt('Ping','Invalid trackback URL or invalid document id',$this->Cnf->L_ERROR);
      return false;
    }

    if( $this->Cnf->CheckAllowSend($fDocID) != $this->ckAllow){
      return false;
    }

    $docObj = $modx->getDocumentObject('id',$fDocID);

    $data['title'] = $docObj['pagetitle'] != '' ? $docObj['pagetitle'] : $_lang['no_pagetitle'];

    if( $docObj['introtext'] != "" ){
      $data['excerpt'] = $modx->stripTags($docObj['introtext']);
    }else{
      $data['excerpt'] = $this->CutText($modx->stripTags($docObj['content']),$this->excerptContentSize,true);
    }
    $data['url'] = $modx->makeUrl($fDocID,'','','full');
    $data['blog_name'] = $modx->config['site_name'];

    // Parse the target
    if( preg_match('/^https?:\/\//',$fTbURL) == 0 ){
      $target = parse_url('http://'.$fTbURL);
    }else{
      $target = parse_url($fTbURL);
    }

    $target["query"] = (isset($target["query"])) && ($target["query"] != "") ? '?' . $target["query"]:'';

    if( ! isset($target["port"]) || ! is_numeric($target["port"])){
      $target["port"] = 80;
    } 
    // Open the socket
    $tb_sock = fsockopen($target["host"], $target["port"]); 
    // Something didn't work out, return
    if (!is_resource($tb_sock)) {
      $this->Cnf->Pnt('Ping','Socket open failed.',$this->Cnf->L_ERROR);
      return false;
      //return '$trackback->ping: can\'t connect to: ' . $tb . '.';
    }
    // Put together the things we want to send
    $tb_send = "url=" . rawurlencode($data['url']) 
      . "&title=" . rawurlencode($data['title']) 
      . "&blog_name=" . rawurlencode($data['blog_name']) 
      . "&excerpt=" . rawurlencode($data['excerpt']); 
    
    // Send the trackback
    fputs($tb_sock, "POST " . $target["path"] . $target["query"] . " HTTP/1.1\r\n");
    fputs($tb_sock, "Host: " . $target["host"] . "\r\n");
    fputs($tb_sock, "Content-type: application/x-www-form-urlencoded\r\n");
    fputs($tb_sock, "Content-length: " . strlen($tb_send) . "\r\n");
    fputs($tb_sock, "Connection: close\r\n\r\n");
    fputs($tb_sock, $tb_send); 
    // Gather result
    while (!feof($tb_sock)) {
      $response .= fgets($tb_sock, 128);
    } 
    // Close socket
    fclose($tb_sock); 

    // Did the trackback ping work
    if( strpos($response, '<error>0</error>') ){
      $this->Cnf->Pnt('Ping','Trackback ping successed:'.$fTbURL,$this->Cnf->L_ERROR);
      return true;
    }

    $this->Cnf->Pnt('Ping','Trackback failed. XML Received:'.$response,$this->Cnf->L_ERROR);
    return false;

  } 

  //-----------------------------------------------------
  //  Search text for links, and searches links for trackback URLs.
  //
  //  $text  : string : document data
  //  return : array  : trackback URLs
  //-----------------------------------------------------
  function URLAutoDiscovery($text){ 
    // Get a list of UNIQUE links from text...
    // ---------------------------------------
    // RegExp to look for (0=>link, 4=>host in 'replace')
    $reg_exp = "/(http)+(s)?:(\\/\\/)((\\w|\\.)+)(\\/)?(\\S+)?/i"; 
    // Make sure each link ends with [sapce]
    $text = eregi_replace("www.", "http://www.", $text);
    $text = eregi_replace("http://http://", "http://", $text);
    $text = eregi_replace("\"", " \"", $text);
    $text = eregi_replace("'", " '", $text);
    $text = eregi_replace(">", " >", $text); 
    // Create an array with unique links
    $uri_array = array();
    if (preg_match_all($reg_exp, strip_tags($text, "<a>"), $array, PREG_PATTERN_ORDER)) {
      foreach($array[0] as $key => $link) {
	foreach((array(",", ".", ":", ";")) as $t_key => $t_value) {
	  $link = trim($link, $t_value);
	} 
	$uri_array[] = ($link);
      } 
      $uri_array = array_unique($uri_array);
    } 
    // Get the trackback URIs from those links...
    // ------------------------------------------
    // Loop through the URIs array and extract RDF segments
    $rdf_array = array(); // <- holds list of RDF segments
    foreach($uri_array as $key => $link) {
      if ($link_content = implode("", @file($link))) {
	preg_match_all('/(<rdf:RDF.*?<\/rdf:RDF>)/sm', $link_content, $link_rdf, PREG_SET_ORDER);
	for ($i = 0; $i < count($link_rdf); $i++) {
	  if (preg_match('|dc:identifier="' . preg_quote($link) . '"|ms', $link_rdf[$i][1])) {
	    $rdf_array[] = trim($link_rdf[$i][1]);
	  } 
	} 
      } 
    } 
    // Loop through the RDFs array and extract trackback URIs
    $tb_array = array(); // <- holds list of trackback URIs
    if (!empty($rdf_array)) {
      for ($i = 0; $i < count($rdf_array); $i++) {
	if (preg_match('/trackback:ping="([^"]+)"/', $rdf_array[$i], $array)) {
	  $tb_array[] = trim($array[1]);
	} 
      } 
    } 
    // Return Trackbacks
    return $tb_array;
  } 

  /**
   * Converts a string into an XML-safe string (replaces &, <, >, " and ')
   * 
   * @param string $string 
   * @return string 
   */
  function xml_safe($string)
  {
    return htmlspecialchars($string, ENT_QUOTES);
  } 

  //-----------------------------------------------------
  //  Response XML messsage for processing trackback.
  //
  //  $status : boolean  : Process success or fail
  //  $msg    : string   : Error messages
  //  return  : string   : XML
  //-----------------------------------------------------
  function MakeResponseXML($status = false, $msg = ""){ 
    // Default error response in case of problems...
    if (!$status && empty($msg)) {
      $msg = "Trackback failed.";
    } 
    // Start response to trackbacker...
    $output = '<?xml version="1.0" encoding="' . $this->Charset . '"?>' . "\n"
      . "<response> \n"; 
    $output .= $status == true ? "  <error>0</error>\n" : "  <error>1</error>\n  <message>" . $this->xml_safe($msg) . "</message>\n";
    $output .= "</response>";

    return $output;
  } 

  //-----------------------------------------------------
  //  Cuts a string short (with text_continue string )
  //
  //  $string : string   : text
  //  $len    : interger : max length
  //  $add    : bool     : add text_continue
  //  return  : string   : cut string
  //-----------------------------------------------------
  function CutText($string,$len=255,$add=false){
    global $_lang;

    if( $this->mbEnable == true ){
      if (mb_strlen($string) > $len) {       
	$tmpstr = mb_substr($string, 0, $len);
	return $add == true ? $tmpstr . $_lang['text_continue']:$tmpstr;
      } 
    }else{
      if (strlen($string) > $len) {
	$tmpstr = substr($string, 0, $len);
	return $add == true ? $tmpstr . $_lang['text_continue']:$tmpstr;
      } 
    }
    return $string;
  } 

}
?>
